@@ -70,8 +70,8 @@ pub fn parser(args: AttributeArgs, input: ItemFn) -> Result<(TokenStream, Functi
         #input
 
         #[doc(hidden)]
-        pub extern "C" fn #internal_ident(ex: &mut ::ext_php_rs::php::execution_data::ExecutionData, retval: &mut ::ext_php_rs::php::types::zval::Zval) {
-            use ::ext_php_rs::php::types::zval::IntoZval;
+        pub extern "C" fn #internal_ident(ex: &mut ::ext_php_rs::zend::ExecuteData, retval: &mut ::ext_php_rs::types::Zval) {
+            use ::ext_php_rs::convert::IntoZval;
 
             #(#arg_definitions)*
             #arg_parser
@@ -79,7 +79,7 @@ pub fn parser(args: AttributeArgs, input: ItemFn) -> Result<(TokenStream, Functi
             let result = #ident(#(#arg_accessors, )*);
 
             if let Err(e) = result.set_zval(retval, false) {
-                let e: ::ext_php_rs::php::exceptions::PhpException = e.into();
+                let e: ::ext_php_rs::exception::PhpException = e.into();
                 e.throw().expect("Failed to throw exception");
             }
         }
@@ -206,7 +206,7 @@ pub fn build_arg_parser<'a>(
                 let this = match this {
                     Some(this) => this,
                     None => {
-                        ::ext_php_rs::php::exceptions::PhpException::default("Failed to retrieve reference to `$this`".into())
+                        ::ext_php_rs::exception::PhpException::default("Failed to retrieve reference to `$this`".into())
                             .throw()
                             .unwrap();
                         return;
@@ -309,7 +309,7 @@ impl Arg {
     pub fn get_type_ident(&self) -> TokenStream {
         let ty: Type = syn::parse_str(&self.ty).unwrap();
         quote! {
-            <#ty as ::ext_php_rs::php::types::zval::FromZval>::TYPE
+            <#ty as ::ext_php_rs::convert::FromZvalMut>::TYPE
         }
     }
 
@@ -318,7 +318,8 @@ impl Arg {
         Ident::new(&self.name, Span::call_site())
     }
 
-    /// Returns a [`TokenStream`] containing the line required to retrieve the value from the argument.
+    /// Returns a [`TokenStream`] containing the line required to retrieve the
+    /// value from the argument.
     pub fn get_accessor(&self, ret: &TokenStream) -> TokenStream {
         let name = &self.name;
         let name_ident = self.get_name_ident();
@@ -338,7 +339,7 @@ impl Arg {
                 match #name_ident.val() {
                     Some(val) => val,
                     None => {
-                        ::ext_php_rs::php::exceptions::PhpException::default(
+                        ::ext_php_rs::exception::PhpException::default(
                             concat!("Invalid value given for argument `", #name, "`.").into()
                         )
                         .throw()
@@ -350,7 +351,8 @@ impl Arg {
         }
     }
 
-    /// Returns a [`TokenStream`] containing the line required to instantiate the argument.
+    /// Returns a [`TokenStream`] containing the line required to instantiate
+    /// the argument.
     pub fn get_arg_definition(&self) -> TokenStream {
         let name = &self.name;
         let ty = self.get_type_ident();
@@ -363,7 +365,7 @@ impl Arg {
         });
 
         quote! {
-            ::ext_php_rs::php::args::Arg::new(#name, #ty) #null #default
+            ::ext_php_rs::args::Arg::new(#name, #ty) #null #default
         }
     }
 }
@@ -397,12 +399,12 @@ impl Function {
 
             // TODO allow reference returns?
             quote! {
-                .returns(<#ty as ::ext_php_rs::php::types::zval::IntoZval>::TYPE, false, #nullable)
+                .returns(<#ty as ::ext_php_rs::convert::IntoZval>::TYPE, false, #nullable)
             }
         });
 
         quote! {
-            ::ext_php_rs::php::function::FunctionBuilder::new(#name, #name_ident)
+            ::ext_php_rs::builders::FunctionBuilder::new(#name, #name_ident)
                 #(#args)*
                 #output
                 .build()
